@@ -864,7 +864,7 @@ GList *poppler_page_find_text_with_options(PopplerPage *page, const char *text,
     xMin = 0;
     yMin = backwards ? height : 0;
 
-    continueMatch.x1 = G_MAXDOUBLE; // we use this to detect valid returned values
+    continueMatch.x1 = std::numeric_limits<double>::max(); // we use this to detect valid returned values
 
     while (text_dev->findText(ucs4, ucs4_len, false, true, // startAtTop, stopAtBottom
                               start_at_last,
@@ -881,7 +881,7 @@ GList *poppler_page_find_text_with_options(PopplerPage *page, const char *text,
         matches = g_list_prepend(matches, match);
         start_at_last = TRUE;
 
-        if (continueMatch.x1 != G_MAXDOUBLE) {
+        if (continueMatch.x1 != std::numeric_limits<double>::max()) {
             // received rect for next-line part of a multi-line match, add it.
             if (multiline) {
                 match->match_continued = true;
@@ -896,7 +896,7 @@ GList *poppler_page_find_text_with_options(PopplerPage *page, const char *text,
                 matches = g_list_prepend(matches, match);
             }
 
-            continueMatch.x1 = G_MAXDOUBLE;
+            continueMatch.x1 = std::numeric_limits<double>::max();
         }
     }
 
@@ -1119,7 +1119,6 @@ static void poppler_page_init(PopplerPage *page) { }
 GList *poppler_page_get_link_mapping(PopplerPage *page)
 {
     GList *map_list = nullptr;
-    gint i;
     Links *links;
     double width, height;
 
@@ -1133,13 +1132,11 @@ GList *poppler_page_get_link_mapping(PopplerPage *page)
 
     poppler_page_get_size(page, &width, &height);
 
-    for (i = 0; i < links->getNumLinks(); i++) {
+    for (AnnotLink *link : links->getLinks()) {
         PopplerLinkMapping *mapping;
         PopplerRectangle rect;
         LinkAction *link_action;
-        AnnotLink *link;
 
-        link = links->getLink(i);
         link_action = link->getAction();
 
         /* Create the mapping */
@@ -1285,7 +1282,6 @@ GList *poppler_page_get_annot_mapping(PopplerPage *page)
 {
     GList *map_list = nullptr;
     double width, height;
-    gint i;
     Annots *annots;
     const PDFRectangle *crop_box;
 
@@ -1299,14 +1295,12 @@ GList *poppler_page_get_annot_mapping(PopplerPage *page)
     poppler_page_get_size(page, &width, &height);
     crop_box = page->page->getCropBox();
 
-    for (i = 0; i < annots->getNumAnnots(); i++) {
+    for (Annot *annot : annots->getAnnots()) {
         PopplerAnnotMapping *mapping;
         PopplerRectangle rect;
-        Annot *annot;
         gboolean flag_no_rotate;
         gint rotation = 0;
 
-        annot = annots->getAnnot(i);
         flag_no_rotate = annot->getFlags() & Annot::flagNoRotate;
 
         /* Create the mapping */
